﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace OnWayDataMaker
{
    public partial class Form1 : Form
    {
        private ShapeFileFeatureLayer layer;
        private InMemoryFeatureLayer editLayer;
        private InMemoryFeatureLayer flagLayer;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            string filePath = @"C:\Program Files\ThinkGeo\Map Suite Routing Evaluation Edition 3.0\HowDoISamples\SampleData\AustinWithOneWayRoad.shp";
            layer = new ShapeFileFeatureLayer(filePath, ShapeFileReadWriteMode.ReadWrite);
            layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.LocalRoad1("FENAME");
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping;
            layer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay overlay = new LayerOverlay();
            overlay.Layers.Add(layer);
            winformsMap1.Overlays.Add(overlay);

            editLayer = new InMemoryFeatureLayer();
            editLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.FromArgb(180, GeoColor.SimpleColors.Red), 8, true);
            editLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            overlay.Layers.Add(editLayer);

            flagLayer = new InMemoryFeatureLayer();
            flagLayer.Open();
            flagLayer.Columns.Add(new FeatureSourceColumn("Flag"));
            ValueStyle valueStyle = new ValueStyle();
            valueStyle.ColumnName = "Flag";

            MemoryStream m1 = new MemoryStream();
            Properties.Resources.startpoint_marker.Save(m1, ImageFormat.Png);
            PointStyle style1 = new PointStyle(new GeoImage(m1));
            valueStyle.ValueItems.Add(new ValueItem("Start", style1));

            MemoryStream m2 = new MemoryStream();
            Properties.Resources.endpoint_marker.Save(m2, ImageFormat.Png);
            PointStyle style2 = new PointStyle(new GeoImage(m2));
            valueStyle.ValueItems.Add(new ValueItem("End", style2));
            flagLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(valueStyle);

            flagLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            overlay.Layers.Add(flagLayer);

            winformsMap1.CurrentExtent = new RectangleShape(-97.7496644339599, 30.3015288337212, -97.7387424788513, 30.2932998164635);
            winformsMap1.ExtentOverlay.DoubleLeftClickMode = MapDoubleLeftClickMode.Disabled;
            winformsMap1.Refresh();
        }

        private void winformsMap1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            PointShape position = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);
            layer.Open();
            Collection<Feature> features = layer.FeatureSource.GetFeaturesNearestTo(position, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.AllColumns);
            if (features.Count > 0)
            {
                editLayer.InternalFeatures.Clear();
                editLayer.InternalFeatures.Add(features[0]);
                // Render start and end flag
                flagLayer.InternalFeatures.Clear();
                LineShape line = ((MultilineShape)features[0].GetShape()).Lines[0];
                Dictionary<string, string> values = new Dictionary<string, string>();
                values.Add("Flag", "Start");
                Feature start = new Feature(new PointShape(line.Vertices[0].X, line.Vertices[0].Y), values);
                flagLayer.InternalFeatures.Add(start);

                Dictionary<string, string> values1 = new Dictionary<string, string>();
                values1.Add("Flag", "End");
                int count = line.Vertices.Count;
                Feature end = new Feature(new PointShape(line.Vertices[count - 1].X, line.Vertices[count - 1].Y), values1);
                flagLayer.InternalFeatures.Add(end);

                winformsMap1.Refresh();

                DialogResult result = MessageBox.Show("Do you want to convert the selected road to one way? Click 'Yes' to set the way direction from start to end, or click 'No' to set the way direction from end to start, and click 'Cancle' to quit.", "Change Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                Feature editFeature = features[0];
                switch (result)
                {
                    case DialogResult.Yes:
                        editFeature.ColumnValues["ONE_WAY"] = "1";
                        editFeature.ColumnValues["INDICATOR"] = "0";
                        UpdateFeature(editFeature);
                        break;
                    case DialogResult.No:
                        editFeature.ColumnValues["ONE_WAY"] = "1";
                        editFeature.ColumnValues["INDICATOR"] = "1";
                        UpdateFeature(editFeature);
                        break;
                    case DialogResult.Cancel:
                    default:
                        break;
                }
            }
        }

        private void UpdateFeature(Feature feature)
        {
            try
            {
                layer.Open();
                layer.FeatureSource.BeginTransaction();
                layer.FeatureSource.UpdateFeature(feature);
                layer.FeatureSource.CommitTransaction();
                MessageBox.Show("Update Successfully!");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Failed update, please try again.\r\n Error Message:\r\n" + ex.Message);
            }
        }
    }
}
